program ConvertCodeLib;

uses
  SysUtils, BDE, DBTables, DBClient, Dialogs, DB, MidasLib;

var
  CDS: TClientDataSet;
  Open: TOpenDialog;
  Save: TSaveDialog;
  Table: TTable;
  NumRecords: Integer;
  OriginalKeys: array of Integer;
  NewKeys: array of Integer;
  i: Integer;
begin
  Open := TOpenDialog.Create(nil);
  Save := TSaveDialog.Create(nil);
  CDS := TClientDataSet.Create(nil);
  Table := TTable.Create(nil);

  Open.Title := 'Select Old Paradox CodeLib Database (CodeDb.db)';
  Open.Filter := 'Paradox Databases (*.db)|*.db';
  Open.DefaultExt := 'db';
  Save.Title := 'Select New CDS Location (CodeDb.cds)';
  Save.Filter := 'Client Datasets (*.cds)|*.cds';
  Save.DefaultExt := 'cds';
  Save.Options := Save.Options + [ofOverwritePrompt];
  Save.FileName := 'CodeDB.cds';

  ShowMessage('This utility converts your Paradox Code Librarian '+
    'database from GExperts 1.0 to the new ClientDataset format used '+
    'in GExperts 1.1 and later.  Steps:'#13#10 +
    '  1. Select your old Paradox format Code Proofreader database'#13#10+
    '  2. Select the filename to save the converted ClientDataset');

  if not (Open.Execute and Save.Execute) then
   Exit;

  Table.DatabaseName := ExtractFilePath(Open.FileName);
  Table.TableName := ExtractFileName(Open.FileName);
  Table.Open;

  NumRecords := Table.RecordCount;
  SetLength(OriginalKeys, NumRecords);
  SetLength(NewKeys, NumRecords);

  with CDS do
  begin
    FileName := Save.FileName;
    FieldDefs.Add('Key', ftAutoInc, 0, False);
    FieldDefs.Add('Type', ftString, 1, True);
    FieldDefs.Add('Parent', ftInteger, 0, False);
    FieldDefs.Add('System', ftInteger, 0, False);
    FieldDefs.Add('Topic', ftString, 50, False);
    FieldDefs.Add('Code', ftMemo, 0, False);
    FieldDefs.Add('Language', ftString, 1, False);
    CreateDataSet;
    LogChanges := False;
    Open;
    AddIndex('Parent', 'Parent', [ixPrimary]);
    AddIndex('Main', 'Key', []);
    AddIndex('System', 'System', []);
    IndexDefs.Update;
    IndexName := 'Parent';
  end;

  i := 0;
  while not Table.Eof do
  begin
    CDS.Insert;
    OriginalKeys[i] := Table.FieldByName('Key').AsInteger;
    CDS.FieldByName('Key').AsInteger := Table.FieldByName('Key').AsInteger;
    CDS.FieldByName('Type').AsString := Table.FieldByName('Type').AsString;
    CDS.FieldByName('Parent').AsInteger := Table.FieldByName('Parent').AsInteger;
    CDS.FieldByName('System').AsInteger := Table.FieldByName('System').AsInteger;
    CDS.FieldByName('Topic').AsString := Table.FieldByName('Topic').AsString;
    CDS.FieldByName('Code').AsString := Table.FieldByName('Code').AsString;
    CDS.FieldByName('Language').AsString := Table.FieldByName('Language').AsString;
    CDS.Post;
    NewKeys[i] := CDS.FieldByName('Key').AsInteger;
    Inc(i);
    Table.Next;
  end;

  for i := 0 to Length(OriginalKeys) - 1 do
  begin
    if NewKeys[i] = OriginalKeys[i] then
      Continue;
    CDS.Filtered := False;
    CDS.Filter := 'Parent=' + IntToStr(OriginalKeys[i]);
    CDS.Filtered := True;
    while not CDS.Eof do
    begin
      CDS.Edit;
      CDS.FieldByName('Parent').AsInteger := NewKeys[i];
      CDS.Post;
    end;
  end;

  CDS.Filtered := False;
  CDS.SaveToFile;
  ShowMessage('Successfuly converted ' + IntToStr(CDS.RecordCount) +
    ' folders and snippets.'#13#10+  'Please copy ' + Save.FileName +
    ' to the directory you have chosen for your Code Librarian database' +
    ' (see the Code Librarian options) to begin using it.');
end.
